@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: bitbang.c,v 1.4 2006/08/23 21:06:28 joerg_wunsch Exp $ */
+/* $Id: bitbang.c,v 1.6 2007/01/24 21:07:54 joerg_wunsch Exp $ */
 
 #include "ac_cfg.h"
 
@@ -33,16 +33,13 @@
 #  include <sys/time.h>
 #endif
 
+#include "avrdude.h"
 #include "avr.h"
 #include "pindefs.h"
 #include "pgm.h"
 #include "par.h"
 #include "serbb.h"
 
-extern char * progname;
-extern int do_cycles;
-extern int verbose;
-
 static int delay_decrement;
 
 #if !defined(WIN32NATIVE)
@@ -355,4 +352,10 @@ void bitbang_check_prerequisites(PROGRAMMER *pgm)
   verify_pin_assigned(pgm, PIN_AVR_SCK,   "AVR SCK");
   verify_pin_assigned(pgm, PIN_AVR_MISO,  "AVR MISO");
   verify_pin_assigned(pgm, PIN_AVR_MOSI,  "AVR MOSI");
+
+  if (pgm->cmd == NULL) {
+    fprintf(stderr, "%s: error: no cmd() method defined for bitbang programmer\n",
+            progname);
+    exit(1);
+  }
 }
