@@ -18,10 +18,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: stk500v2.h,v 1.6 2006/11/01 21:47:25 joerg_wunsch Exp $ */
+/* $Id: stk500v2.h,v 1.7 2007/01/24 22:43:46 joerg_wunsch Exp $ */
 
-#ifndef stk500v2_h__
-#define stk500v2_h__
+#ifndef stk500v2_h
+#define stk500v2_h
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 void stk500v2_initpgm (PROGRAMMER * pgm);
 void stk500hvsp_initpgm (PROGRAMMER * pgm);
@@ -31,6 +35,10 @@ void stk500v2_dragon_hvsp_initpgm(PROGRAMMER * pgm);
 void stk500v2_dragon_isp_initpgm(PROGRAMMER * pgm);
 void stk500v2_dragon_pp_initpgm(PROGRAMMER * pgm);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
 
