@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: term.c,v 1.35 2004/12/22 01:52:45 bdean Exp $ */
+/* $Id: term.c,v 1.38 2007/01/24 22:43:46 joerg_wunsch Exp $ */
 
 #include "ac_cfg.h"
 
@@ -34,6 +34,7 @@
 #endif
 #endif
 
+#include "avrdude.h"
 #include "avr.h"
 #include "config.h"
 #include "lists.h"
@@ -41,12 +42,6 @@
 #include "pindefs.h"
 #include "ppi.h"
 
-
-extern char       * progname;
-extern char         progbuf[];
-extern PROGRAMMER * pgm;
-
-
 struct command {
   char * name;
   int (*func)(PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
@@ -54,31 +49,44 @@ struct command {
 };
 
 
-int cmd_dump  (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_dump  (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_write (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_write (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_erase (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_erase (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_sig   (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_sig   (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_part  (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_part  (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_help  (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_help  (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_quit  (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_quit  (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_send  (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_send  (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_parms (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_parms (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_vtarg (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_vtarg (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_varef (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_varef (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_fosc  (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_fosc  (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
-int cmd_sck   (PROGRAMMER * pgm, struct avrpart * p, int argc, char *argv[]);
+static int cmd_sck   (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
 
 
 struct command cmd[] = {
@@ -105,7 +113,7 @@ struct command cmd[] = {
 
 
 
-int nexttok(char * buf, char ** tok, char ** next)
+static int nexttok(char * buf, char ** tok, char ** next)
 {
   char * q, * n;
 
@@ -134,7 +142,7 @@ int nexttok(char * buf, char ** tok, char ** next)
 }
 
 
-int hexdump_line(char * buffer, unsigned char * p, int n, int pad)
+static int hexdump_line(char * buffer, unsigned char * p, int n, int pad)
 {
   char * hexdata = "0123456789abcdef";
   char * b;
@@ -166,7 +174,7 @@ int hexdump_line(char * buffer, unsigned char * p, int n, int pad)
 }
 
 
-int chardump_line(char * buffer, unsigned char * p, int n, int pad)
+static int chardump_line(char * buffer, unsigned char * p, int n, int pad)
 {
   int i;
   char b [ 128 ];
@@ -189,7 +197,7 @@ int chardump_line(char * buffer, unsigned char * p, int n, int pad)
 }
 
 
-int hexdump_buf(FILE * f, int startaddr, unsigned char * buf, int len)
+static int hexdump_buf(FILE * f, int startaddr, unsigned char * buf, int len)
 {
   int addr;
   int i, n;
@@ -216,7 +224,8 @@ int hexdump_buf(FILE * f, int startaddr, unsigned char * buf, int len)
 }
 
 
-int cmd_dump(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_dump(PROGRAMMER * pgm, struct avrpart * p,
+		    int argc, char * argv[])
 {
   static char prevmem[128] = {0};
   char * e;
@@ -292,7 +301,7 @@ int cmd_dump(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
   }
 
   for (i=0; i<len; i++) {
-    rc = avr_read_byte(pgm, p, mem, addr+i, &buf[i]);
+    rc = pgm->read_byte(pgm, p, mem, addr+i, &buf[i]);
     if (rc != 0) {
       fprintf(stderr, "error reading %s address 0x%05lx of part %s\n",
               mem->desc, addr+i, p->desc);
@@ -315,7 +324,8 @@ int cmd_dump(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_write(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_write(PROGRAMMER * pgm, struct avrpart * p,
+		     int argc, char * argv[])
 {
   char * e;
   int len, maxsize;
@@ -399,7 +409,7 @@ int cmd_write(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
       werror = 1;
     }
 
-    rc = avr_read_byte(pgm, p, mem, addr+i, &b);
+    rc = pgm->read_byte(pgm, p, mem, addr+i, &b);
     if (b != buf[i]) {
       fprintf(stderr, 
               "%s (write): error writing 0x%02x at 0x%05lx cell=0x%02x\n",
@@ -420,13 +430,21 @@ int cmd_write(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_send(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_send(PROGRAMMER * pgm, struct avrpart * p,
+		    int argc, char * argv[])
 {
   unsigned char cmd[4], res[4];
   char * e;
   int i;
   int len;
 
+  if (pgm->cmd == NULL) {
+    fprintf(stderr,
+	    "The %s programmer does not support direct ISP commands.\n",
+	    pgm->type);
+    return -1;
+  }
+
   if (argc != 5) {
     fprintf(stderr, "Usage: send <byte1> <byte2> <byte3> <byte4>\n");
     return -1;
@@ -463,7 +481,8 @@ int cmd_send(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_erase(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_erase(PROGRAMMER * pgm, struct avrpart * p,
+		     int argc, char * argv[])
 {
   fprintf(stderr, "%s: erasing chip\n", progname);
   pgm->chip_erase(pgm, p);
@@ -471,7 +490,8 @@ int cmd_erase(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_part(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_part(PROGRAMMER * pgm, struct avrpart * p,
+		    int argc, char * argv[])
 {
   fprintf(stdout, "\n");
   avr_display(stdout, p, "", 0);
@@ -481,7 +501,8 @@ int cmd_part(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_sig(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_sig(PROGRAMMER * pgm, struct avrpart * p,
+		   int argc, char * argv[])
 {
   int i;
   int rc;
@@ -510,13 +531,15 @@ int cmd_sig(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_quit(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_quit(PROGRAMMER * pgm, struct avrpart * p,
+		    int argc, char * argv[])
 {
   return 1;
 }
 
 
-int cmd_parms(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_parms(PROGRAMMER * pgm, struct avrpart * p,
+		     int argc, char * argv[])
 {
   if (pgm->print_parms == NULL) {
     fprintf(stderr,
@@ -531,7 +554,8 @@ int cmd_parms(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_vtarg(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_vtarg(PROGRAMMER * pgm, struct avrpart * p,
+		     int argc, char * argv[])
 {
   int rc;
   double v;
@@ -561,7 +585,8 @@ int cmd_vtarg(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_fosc(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_fosc(PROGRAMMER * pgm, struct avrpart * p,
+		    int argc, char * argv[])
 {
   int rc;
   double v;
@@ -600,7 +625,8 @@ int cmd_fosc(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_sck(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_sck(PROGRAMMER * pgm, struct avrpart * p,
+		   int argc, char * argv[])
 {
   int rc;
   double v;
@@ -632,7 +658,8 @@ int cmd_sck(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_varef(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_varef(PROGRAMMER * pgm, struct avrpart * p,
+		     int argc, char * argv[])
 {
   int rc;
   double v;
@@ -662,7 +689,8 @@ int cmd_varef(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int cmd_help(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int cmd_help(PROGRAMMER * pgm, struct avrpart * p,
+		    int argc, char * argv[])
 {
   int i;
 
@@ -680,7 +708,7 @@ int cmd_help(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
 }
 
 
-int tokenize(char * s, char *** argv)
+static int tokenize(char * s, char *** argv)
 {
   int     i, n, l, nargs, offset;
   int     len, slen;
@@ -757,7 +785,8 @@ int tokenize(char * s, char *** argv)
 }
 
 
-int do_cmd(PROGRAMMER * pgm, struct avrpart * p, int argc, char * argv[])
+static int do_cmd(PROGRAMMER * pgm, struct avrpart * p,
+		  int argc, char * argv[])
 {
   int i;
   int hold;
